#ifndef MSP_GRAPHICS_GLCONTEXT_H_
#define MSP_GRAPHICS_GLCONTEXT_H_

#include <stdexcept>

namespace Msp {
namespace Graphics {

class Display;
class Window;

struct GLOptions
{
	enum
	{
		DEFAULT_VERSION = 0,
		LATEST_VERSION = 0xFFFFFFFF
	};

	bool alpha;
	bool stencil;
	bool doublebuffer;
	unsigned multisample;
	bool forward_compatible;
	bool core_profile;
	unsigned gl_version_major;
	unsigned gl_version_minor;

	GLOptions();
};


class unsupported_gl_mode: public std::runtime_error
{
public:
	unsupported_gl_mode(const GLOptions &);
	virtual ~unsupported_gl_mode() throw () { }

private:
	static std::string format_version(const GLOptions &);
};


class GLContext
{
private:
	struct Private;

	Display &display;
	Window &window;
	Private *priv;

public:
	GLContext(Window &wnd, const GLOptions &opts = GLOptions());
	GLContext(Window &wnd, unsigned, unsigned);
private:
	void platform_init(const GLOptions &);
public:
	~GLContext();

	void set_swap_interval(unsigned);

	void swap_buffers();
private:
	void window_resized(unsigned, unsigned);
};

} // namespace Graphics
} // namespace Msp

#endif
