#ifndef MSP_GRAPHICS_IMAGELOADER_H_
#define MSP_GRAPHICS_IMAGELOADER_H_

#include "image.h"

namespace Msp {
namespace Graphics {

class unsupported_image_format: public std::runtime_error
{
public:
	unsupported_image_format(const std::string &w): std::runtime_error(w) { }
	virtual ~unsupported_image_format() throw() { }
};

class bad_image_data: public std::runtime_error
{
public:
	bad_image_data(const std::string &w): std::runtime_error(w) { }
	virtual ~bad_image_data() throw() { }
};


class ImageLoader
{
protected:
	class RegisterBase
	{
	protected:
		RegisterBase() { }
	public:
		virtual ~RegisterBase() { }

		virtual unsigned get_signature_size() const = 0;
		virtual bool detect(const std::string &) const = 0;
		virtual ImageLoader *create(IO::Seekable &) const = 0;
	};

	template<typename T>
	class RegisteredLoader: public RegisterBase
	{
	public:
		virtual unsigned get_signature_size() const { return T::get_signature_size(); }
		virtual bool detect(const std::string &s) const { return T::detect(s); }
		virtual ImageLoader *create(IO::Seekable &io) const { return new T(io); }
	};

	struct Registry
	{
		std::list<RegisterBase *> loaders;
		bool changed;

		Registry();
		~Registry();
	};

private:
	IO::Base *source;

protected:
	ImageLoader();
public:
	virtual ~ImageLoader();

	static ImageLoader *open_file(const std::string &);
	static ImageLoader *open_io(IO::Seekable &);

	virtual void load(Image::Data &) = 0;

	template<typename T>
	static void register_loader();
private:
	static Registry &get_registry();

	static bool signature_size_compare(RegisterBase *, RegisterBase *);
};

template<typename T>
void ImageLoader::register_loader()
{
	Registry &registry = get_registry();
	registry.loaders.push_back(new RegisteredLoader<T>);
	registry.changed = true;
}

} // namespace Graphics
} // namespace Msp

#endif
